"use strict";
import * as gtxTool from "./gtx";
// eslint-disable-next-line @typescript-eslint/no-var-requires
const secp256k1 = require("secp256k1");
import * as logger from "../logger";
import { removeDuplicateSigners } from "../formatter";
export function createClient(restApiClient, blockchainRID, functionNames) {
    functionNames.push("message");
    function transaction(gtx) {
        return {
            gtx,
            sign: function (privKey, pubKey) {
                let pub = pubKey;
                if (!pub) {
                    logger.debug(`pubKey not provided, will instead be generated using secp256k1`);
                    pub = Buffer.from(secp256k1.publicKeyCreate(privKey));
                }
                logger.debug(`signing transaction with privKey: ${privKey}, pubKey: ${pub}`);
                gtxTool.sign(privKey, pub, this.gtx);
            },
            getTxRID: function () {
                return this.getDigestToSign();
            },
            getDigestToSign: function () {
                return gtxTool.getDigestToSign(this.gtx);
            },
            addSignature: function (pubKey, signature) {
                gtxTool.addSignature(pubKey, signature, this.gtx);
            },
            // raw call
            addOperation: function (name, ...args) {
                gtxTool.addTransactionToGtx(name, args, this.gtx);
            },
            postAndWaitConfirmation() {
                return restApiClient.postAndWaitConfirmation(gtxTool.serialize(this.gtx), this.getTxRID());
            },
            send: function (callback) {
                const gtxBytes = gtxTool.serialize(this.gtx);
                restApiClient.postTransaction(gtxBytes, callback);
                this.gtx = null;
                this.gtxBytes = gtxBytes;
            },
            encode: function () {
                return gtxTool.serialize(this.gtx);
            },
        };
    }
    function addFunctions(req) {
        functionNames.forEach((functionName) => {
            req[functionName] = function (...args) {
                gtxTool.addTransactionToGtx(functionName, args, this.gtx);
            };
        });
    }
    const client = {
        newTransaction: function (signers) {
            signers = removeDuplicateSigners(signers);
            const newGtx = gtxTool.emptyGtx(Buffer.from(blockchainRID, "hex"));
            signers.forEach((signer) => gtxTool.addSignerToGtx(signer, newGtx));
            const req = transaction(newGtx);
            addFunctions(req);
            return req;
        },
        transactionFromRawTransaction: function (rawTransaction) {
            const gtx = gtxTool.deserialize(rawTransaction);
            logger.debug(`Output from deserializing a raw transaction: ${JSON.stringify(gtx)}`);
            const req = transaction(gtx);
            addFunctions(req);
            return req;
        },
        query: function (queryObject, callback) {
            return restApiClient.query(queryObject, callback);
        },
    };
    return client;
}
//# sourceMappingURL=gtxclient.js.map